! This file is part of HiggsBounds
!  -KW
!******************************************************************
module extra_bits_for_SLHA
!******************************************************************

 contains
      
 !************************************************************      
 subroutine getSLHAdata(d,gsq,infile)
 ! At the moment, this is not a supported part of HiggsBounds
 !(if you would find this useful, drop us an email and we'll
 ! put this feature higher on out priority list)
 ! looks at stuff needed for Higgs searches only
 !************************************************************ 
  use usefulbits, only : dataset,tevextras,Hneut,Hplus,Chineut,Chiplus,np,sqcouplratio,file_id_common
  use SLHA_manip
  use PDGnumbering
  
  implicit none
  !--------------------------------------input
  type(dataset) :: d
  type(sqcouplratio) :: gsq
  character(len=80),intent(in) :: infile
  !-----------------------------------internal
  integer :: i,j,x,ios,n
  integer :: h(5)
  integer :: particlecontent,Rparityviolation,CPviolation
  double precision,allocatable ::g2hjcc(:,:),g2hjss(:,:),g2hjbb(:,:),g2hjtoptop(:,:),g2hjtautau(:,:)
  double precision :: lspcandidate_mass,mass
  integer :: visible_lspcandidate_PDG(21),invisible_lspcandidate_PDG(7)
  !-------------------------------------------              
 
  if((np(Hneut).lt.0).or.(np(Hneut).gt.5))then
    stop'can not use subroutine getSLHAdata if number of neutral Higgs is not in range 0:5'
  endif
  if((np(Hplus).lt.0).or.(np(Hplus).gt.1))then
    stop'can not use subroutine getSLHAdata if number of charged Higgs is not in range 0:1'
  endif
  if((np(Hneut).eq.0).and.(np(Hneut).eq.0))stop'can not use subroutine getSLHAdata if number of Higgs = 0'

  allocate(g2hjcc(    np(Hneut),2))
  allocate(g2hjss(    np(Hneut),2))
  allocate(g2hjbb(    np(Hneut),2))
  allocate(g2hjtoptop(np(Hneut),2))
  allocate(g2hjtautau(np(Hneut),2))

  h(1)=h0 !lightest CP even Higgs in real MSSM
  h(2)=HH !heaviest CP even Higgs in real MSSM
  h(3)=A0 !CP odd Higgs in real MSSM
  h(4)=h03 !NMSSM only
  h(5)=A02 !NMSSM only

  open(file_id_common,file=trim(infile),status='old',action='read',iostat=ios) 
  if(ios.ne.0)then 
   write(*,*)'tried to open $'//trim(adjustl(infile))//'$'
   stop'problem opening SLHA input file'
  endif

  call readSLHAfile(file_id_common)

  particlecontent  =get_modsel(3)
  Rparityviolation =get_modsel(4)
  CPviolation      =get_modsel(5)

  if(Rparityviolation.ne.0)stop'HB can not yet use SLHA files with R parity violation'

  select case(particlecontent)
  case(0)
   if((np(Hneut).ne.0).and.(np(Hneut).ne.3))then
    write(*,*)'warning: modsel(3)=0 (MSSM) in SLHA file'
    write(*,*)' but number of neutral Higgs that HB was told to expect=',np(Hneut)
   endif
  case(1)
   if((np(Hneut).ne.0).and.(np(Hneut).ne.5))then
    write(*,*)'warning: modsel(3)=1 (NMSSM) in SLHA file'
    write(*,*)' but number of neutral Higgs that HB was told to expect=',np(Hneut)
   endif
  end select
  if((np(Chineut).ne.0).or.(np(Chiplus).ne.0))then
    write(*,*)'warning: currently, can only use subroutine getSLHAdata for Higgs sector'
  endif

  if(np(Hneut).gt.0)then
   do i=1,np(Hneut)
    d%particle(Hneut)%M(i) = get_mass( h(i))
    d%particle(Hneut)%GammaTot(i) = get_totaldecaywidth( h(i))

    d%BR_hjss(i) = get_twobodybranchingratio( h(i),squark,sbar)
    d%BR_hjcc(i) = get_twobodybranchingratio( h(i),cquark,cbar)
    d%BR_hjbb(i) = get_twobodybranchingratio( h(i),bquark,bbar)

    d%BR_hjtautau(i) = get_twobodybranchingratio( h(i),taup,taum)

    d%BR_hjWW(i)   = get_twobodybranchingratio( h(i),Wp ,Wm)
    d%BR_hjZZ(i)   = get_twobodybranchingratio( h(i),Z0 ,Z0)
    d%BR_hjZga(i)  = get_twobodybranchingratio( h(i),Z0    ,photon)
    d%BR_hjgaga(i) = get_twobodybranchingratio( h(i),photon,photon)
    d%BR_hjgg(i)   = get_twobodybranchingratio( h(i),gluon ,gluon)

    !----- finding invisible Higgs Br-------
    call fill_visible_lspcandidate_PDG
    call fill_invisible_lspcandidate_PDG

    d%BR_hjinvisible(i) = 0.0D0
    lspcandidate_mass=1.0D12 !set to a very big value

    !find the invisible LSP candidate with the lowest mass
    !and use it for BR_hjinvisible
    do x=1,ubound(invisible_lspcandidate_PDG,dim=1)
     n=invisible_lspcandidate_PDG(x)
     mass=get_mass( n )
     if(mass.gt.0.0D0.and.(mass.lt.lspcandidate_mass))then !mass<0 means that this particle mass couldn't be found
       lspcandidate_mass=mass
       d%BR_hjinvisible(i) = get_twobodybranchingratio( h(i), n, n )
     endif
    enddo

    !however, if there's a charged SUSY particle or gluino with
    !lower mass than the invisible LSP candidate, 
    !this candidate is not the LSP, so set BR_hjinvisible=0
    do x=1,ubound(visible_lspcandidate_PDG,dim=1)
     n=visible_lspcandidate_PDG(x)
     mass=get_mass( n )
     if(mass.gt.0.0D0.and.(mass.lt.lspcandidate_mass))then !mass<0 means that this particle mass couldn't be found
       d%BR_hjinvisible(i) = 0.0D0
     endif
    enddo
    !--------------------------------------

    g2hjcc(i,:)      = get_HiggsCouplingsFermions( h(i), cquark,  cquark   )
    g2hjss(i,:)      = get_HiggsCouplingsFermions( h(i), squark,  squark   )
    g2hjbb(i,:)      = get_HiggsCouplingsFermions( h(i), bquark,  bquark   )
    g2hjtoptop(i,:)  = get_HiggsCouplingsFermions( h(i), tquark,  tquark   )
    g2hjtautau(i,:)  = get_HiggsCouplingsFermions( h(i), taum,    taum     )
    gsq%hjWW(i)      = get_HiggsCouplingsBosons( h(i), Wp,      Wp       )
    gsq%hjZZ(i)      = get_HiggsCouplingsBosons( h(i), Z0,      Z0       )
    gsq%hjgg(i)      = get_HiggsCouplingsBosons( h(i), gluon,   gluon    )

   enddo

   do j=1,np(Hneut)
    do i=1,np(Hneut)
      gsq%hjhiZ(j,i)   = get_HiggsCouplingsBosons( h(j),   h(i),   Z0    )
      d%BR_hjhihi(j,i) = get_twobodybranchingratio( h(j), h(i), h(i) )
    enddo
   enddo
  endif

  if(np(Hplus).gt.0)then
   i=1
   d%particle(Hplus)%M(i) = get_mass( Hp )
   d%particle(Hplus)%GammaTot(i) = get_totaldecaywidth( Hp )
   d%BR_tWpb        = get_twobodybranchingratio( tquark, Wp,   bquark )
   d%BR_tHpjb(i)    = get_twobodybranchingratio( tquark, Hp,   bquark )
   d%BR_Hpjcs(i)    = get_twobodybranchingratio( Hp,     cquark,  sbar   )
   d%BR_Hpjcb(i)    = get_twobodybranchingratio( Hp,     cquark,  bbar   )
   d%BR_Hpjtaunu(i) = get_twobodybranchingratio( Hp,     taup,    nutau  )
  endif

  gsq%hjZga          =0.0D0 !these are not needed
  gsq%hjgaga         =0.0D0

  gsq%hjcc_s(:)          = g2hjcc(:,1)
  gsq%hjcc_p(:)          = g2hjcc(:,2)

  gsq%hjss_s(:)          = g2hjss(:,1)
  gsq%hjss_p(:)          = g2hjss(:,2)

  gsq%hjbb_s(:)          = g2hjbb(:,1)
  gsq%hjbb_p(:)          = g2hjbb(:,2)
 
  gsq%hjtoptop_s(:)      = g2hjtoptop(:,1)
  gsq%hjtoptop_p(:)      = g2hjtoptop(:,2)
 
  gsq%hjtautau_s(:)      = g2hjtautau(:,1)
  gsq%hjtautau_p(:)      = g2hjtautau(:,2)

  deallocate(g2hjcc)
  deallocate(g2hjss)
  deallocate(g2hjbb)
  deallocate(g2hjtoptop)
  deallocate(g2hjtautau)

  close(file_id_common)
  call finishwithSLHA

  contains

  subroutine fill_invisible_lspcandidate_PDG

   invisible_lspcandidate_PDG(1) = neut1

   invisible_lspcandidate_PDG(2) = s_nueL   
   invisible_lspcandidate_PDG(3) = s_numuL  
   invisible_lspcandidate_PDG(4) = s_nutauL 

   invisible_lspcandidate_PDG(5) = s_nutau1A
   invisible_lspcandidate_PDG(6) = s_nutau2A
   invisible_lspcandidate_PDG(7) = s_nutau3A

  end subroutine fill_invisible_lspcandidate_PDG

  subroutine fill_visible_lspcandidate_PDG
  
   visible_lspcandidate_PDG(1) = char1p
   visible_lspcandidate_PDG(2) = char2p
 
   visible_lspcandidate_PDG(3) = s_t1 
   visible_lspcandidate_PDG(4) = s_b1 
   visible_lspcandidate_PDG(5) = s_cL 
   visible_lspcandidate_PDG(6) = s_sL 
   visible_lspcandidate_PDG(7) = s_uL 
   visible_lspcandidate_PDG(8) = s_dL 

   visible_lspcandidate_PDG(9) = s_t2 
   visible_lspcandidate_PDG(10) = s_b2 
   visible_lspcandidate_PDG(11) = s_cR 
   visible_lspcandidate_PDG(12) = s_sR 
   visible_lspcandidate_PDG(13) = s_uR 
   visible_lspcandidate_PDG(14) = s_dR 

   visible_lspcandidate_PDG(15) = s_emL   
   visible_lspcandidate_PDG(16) = s_mumL  
   visible_lspcandidate_PDG(17) = s_taumL  

   visible_lspcandidate_PDG(18) = s_emR    
   visible_lspcandidate_PDG(19) = s_mumR  
   visible_lspcandidate_PDG(20) = s_taumR 

   visible_lspcandidate_PDG(21) = gluino

  end subroutine fill_visible_lspcandidate_PDG


 end subroutine getSLHAdata
 !************************************************************     
 subroutine outputSLHAdata(infile) 
 !************************************************************ 
  use usefulbits, only : res,whichanalyses,pr,vers,file_id_common
  use SLHA_manip
  use S95tables      
  
  implicit none
  !--------------------------------------input
  character(len=80),intent(in) :: infile
  !-----------------------------------internal
  integer :: x,y,ios,jj
  integer :: k_out
  character(LEN=200):: descrip 
  !-------------------------------------------              

  open(file_id_common,file=trim(infile),status='old',iostat=ios) 
  if(ios.ne.0)then 
   write(*,*)'tried to open $'//trim(adjustl(infile))//'$'
   stop'problem opening SLHA input file'
  endif

  !k_out=22
  !open(k_out,file='temp.out')   
  k_out=file_id_common

  call readSLHAfile(file_id_common)
  rewind(file_id_common)
  call writeSLHAfile_exceptblock('HiggsBoundsResults',k_out)

  !write(k_out,'(a)')'#'
  write(k_out,'(a)')'Block HiggsBoundsResults      # results from HiggsBounds http://www.ippp.dur.ac.uk/HiggsBounds'
  write(k_out,'(a)')'# HBresult   : scenario allowed flag (1: allowed, 0: excluded, -1: unphysical)'      
  write(k_out,'(a)')'# chan id number: most sensitive channel (see below). chan=0 if no channel applies'          
  write(k_out,'(a)')'# obsratio   : ratio [sig x BR]_model/[sig x BR]_limit (<1: allowed, >1: excluded)' 
  write(k_out,'(a)')'# ncomb      : number of Higgs bosons combined in most sensitive channel'
  write(k_out,'(a)')'# Note that the HB channel id number varies depending on the HB version and setting "whichanalyses"'
  write(k_out,'(a)')'#'
  write(k_out,*)'   0    '//trim(adjustl(vers))//'     ||'//whichanalyses// &
     &  '||            # version of HB used to produce these results,the HB setting "whichanalyses"'
  write(k_out,'(a)')'#'

  jj=1
  y=1
  x=res(jj)%chan(y)
  call outputproc(pr(x),0,descrip,1)

  write(k_out,'(a)')'#CHANNELTYPE 1: channel with the highest statistical sensitivity'
  write(k_out,*)'   1',1   ,    res(jj)%chan(y),     '                 # channel id number'
  write(k_out,*)'   1',2   ,    res(jj)%allowed95(y),'                 # HBresult      '
  write(k_out,*)'   1',3   ,    res(jj)%obsratio(y), '   # obsratio  '
  write(k_out,*)'   1',4   ,    res(jj)%ncombined(y),'                 # ncombined'      
  write(k_out,*)'   1',5   ,'||'//trim(adjustl(descrip))//'|| # text description of channel'
  write(k_out,'(a)')'#'

  close(file_id_common)
  if(k_out.ne.file_id_common)close(k_out)

  call finishwithSLHA

 end subroutine outputSLHAdata
 !************************************************************ 


end module extra_bits_for_SLHA
!******************************************************************
